@@ -1,9 +1,7 @@
 # Copyright (C) 2011-2012  Patrick Totzke <patricktotzke@gmail.com>
 # This file is released under the GNU GPL, version 3 or a later revision.
 # For further details see the COPYING file
-from __future__ import absolute_import
-
-import imp
+import importlib.util
 import itertools
 import logging
 import mailcap
@@ -31,55 +29,46 @@ DATA_DIRS = get_xdg_env('XDG_DATA_DIRS',
 
 class SettingsManager(object):
     """Organizes user settings"""
-    def __init__(self, alot_rc=None, notmuch_rc=None):
-        """
-        :param alot_rc: path to alot's config file
-        :type alot_rc: str
-        :param notmuch_rc: path to notmuch's config file
-        :type notmuch_rc: str
-        """
-        assert alot_rc is None or (isinstance(alot_rc, basestring) and
-                                   os.path.exists(alot_rc))
-        assert notmuch_rc is None or (isinstance(notmuch_rc, basestring) and
-                                      os.path.exists(notmuch_rc))
+    def __init__(self):
         self.hooks = None
         self._mailcaps = mailcap.getcaps()
         self._notmuchconfig = None
         self._theme = None
         self._accounts = None
         self._accountmap = None
-        self.alot_rc_path = alot_rc
-        self.notmuch_rc_path = notmuch_rc
         self._notmuchconfig = None
         self._config = ConfigObj()
         self._bindings = None
-        self.reload()
 
     def reload(self):
-        """Reload All configurations.
+        """Reload notmuch and alot config files"""
+        self.read_notmuch_config(self._notmuchconfig.filename)
+        self.read_config(self._config.filename)
 
-        This first resets all configs to default (in case an overwritten
-        binding is removed from the user config), then reloads the notmuch
-        config, and finally reads the alot config.
-
-        Implementation Detail: this is the same code called by the constructor
-        to set bindings at alot startup.
+    def read_notmuch_config(self, path):
         """
+        parse notmuch's config file
+        :param path: path to notmuch's config file
+        :type path: str
+        """
+        spec = os.path.join(DEFAULTSPATH, 'notmuch.rc.spec')
+        self._notmuchconfig = read_config(path, spec)
+
+    def _update_bindings(self, newbindings):
+        assert isinstance(newbindings, Section)
+
         self._bindings = ConfigObj(os.path.join(DEFAULTSPATH,
                                                 'default.bindings'))
-        self.read_notmuch_config()
-        self.read_config()
+        self._bindings.merge(newbindings)
 
-    def read_notmuch_config(self):
-        """parse notmuch's config file from path"""
-        spec = os.path.join(DEFAULTSPATH, 'notmuch.rc.spec')
-        self._notmuchconfig = read_config(self.notmuch_rc_path, spec)
-
-    def read_config(self):
-        """parse alot's config file from path"""
+    def read_config(self, path):
+        """
+        parse alot's config file
+        :param path: path to alot's config file
+        :type path: str
+        """
         spec = os.path.join(DEFAULTSPATH, 'alot.rc.spec')
-        newconfig = read_config(
-            self.alot_rc_path, spec, report_extra=True, checks={
+        newconfig = read_config(path, spec, report_extra=True, checks={
                 'mail_container': checks.mail_container,
                 'force_list': checks.force_list,
                 'align': checks.align_mode,
@@ -90,13 +79,13 @@ class SettingsManager(object):
 
         hooks_path = os.path.expanduser(self._config.get('hooksfile'))
         try:
-            self.hooks = imp.load_source('hooks', hooks_path)
+            spec = importlib.util.spec_from_file_location('hooks', hooks_path)
+            self.hooks = importlib.util.module_from_spec(spec)
+            spec.loader.exec_module(self.hooks)
         except:
-            logging.debug('unable to load hooks file:%s', hooks_path)
+            logging.exception('unable to load hooks file:%s', hooks_path)
         if 'bindings' in newconfig:
-            newbindings = newconfig['bindings']
-            if isinstance(newbindings, Section):
-                self._bindings.merge(newbindings)
+            self._update_bindings(newconfig['bindings'])
 
         tempdir = self._config.get('template_dir')
         logging.debug('template directory: `%s`' % tempdir)
@@ -176,12 +165,12 @@ class SettingsManager(object):
 
         value = section[key]
 
-        if isinstance(value, (str, unicode)):
+        if isinstance(value, str):
             section[key] = expand_environment_and_home(value)
         elif isinstance(value, (list, tuple)):
             new = list()
             for item in value:
-                if isinstance(item, (str, unicode)):
+                if isinstance(item, str):
                     new.append(expand_environment_and_home(item))
                 else:
                     new.append(item)
@@ -395,7 +384,7 @@ class SettingsManager(object):
     def get_mapped_input_keysequences(self, mode='global', prefix=u''):
         # get all bindings in this mode
         globalmaps, modemaps = self.get_keybindings(mode)
-        candidates = globalmaps.keys() + modemaps.keys()
+        candidates = list(globalmaps.keys()) + list(modemaps.keys())
         if prefix is not None:
             prefixes = prefix + ' '
             cand = [c for c in candidates if c.startswith(prefixes)]
@@ -433,7 +422,7 @@ class SettingsManager(object):
                 if value and value != '':
                     globalmaps[key] = value
         # get rid of empty commands left in mode bindings
-        for k, v in modemaps.items():
+        for k, v in list(modemaps.items()):
             if not v:
                 del modemaps[k]
 
@@ -475,21 +464,21 @@ class SettingsManager(object):
         """
         return self._accounts
 
-    def get_account_by_address(self, address, return_default=False):
+    def account_matching_address(self, address, return_default=False):
         """returns :class:`Account` for a given email address (str)
 
         :param str address: address to look up. A realname part will be ignored.
         :param bool return_default: If True and no address can be found, then
-            the default account wil be returned
+            the default account wil be returned.
         :rtype: :class:`Account`
         :raises ~alot.settings.errors.NoMatchingAccount: If no account can be
             found. This includes if return_default is True and there are no
             accounts defined.
         """
         _, address = email.utils.parseaddr(address)
-        for myad in self.get_addresses():
-            if myad == address:
-                return self._accountmap[myad]
+        for account in self.get_accounts():
+            if account.matches_address(address):
+                return account
         if return_default:
             try:
                 return self.get_accounts()[0]
@@ -502,10 +491,6 @@ class SettingsManager(object):
         """returns addresses of known accounts without its aliases"""
         return [a.address for a in self._accounts]
 
-    def get_addresses(self):
-        """returns addresses of known accounts including all their aliases"""
-        return self._accountmap.keys()
-
     def get_addressbooks(self, order=None, append_remaining=True):
         """returns list of all defined :class:`AddressBook` objects"""
         order = order or []
@@ -529,7 +514,7 @@ class SettingsManager(object):
 
     def represent_datetime(self, d):
         """
-        turns a given datetime obj into a unicode string representation.
+        turns a given datetime obj into a string representation.
         This will:
 
         1) look if a fixed 'timestamp_format' is given in the config
