@@ -3,12 +3,12 @@
 # Copyright © 2017 Dylan Baker
 # This file is released under the GNU GPL, version 3 or a later revision.
 # For further details see the COPYING file
-from __future__ import absolute_import
-
 import base64
+import codecs
 import email
 import email.header
 import email.mime.application
+import email.policy
 import io
 import os
 import os.path
@@ -20,7 +20,6 @@ import gpg
 import mock
 
 from alot import crypto
-from alot import helper
 from alot.db import utils
 from alot.errors import GPGProblem
 from ..utilities import make_key, make_uid, TestCaseClassCleanup
@@ -164,71 +163,6 @@ class TestExtractHeader(unittest.TestCase):
         self.assertEqual(actual, expected)
 
 
-class TestEncodeHeader(unittest.TestCase):
-
-    def test_only_value_is_used_in_output(self):
-        actual = utils.encode_header('x-key', 'value')
-        expected = email.header.Header('value')
-        self.assertEqual(actual, expected)
-
-    def test_unicode_chars_are_encoded(self):
-        actual = utils.encode_header('x-key', u'välüe')
-        expected = email.header.Header('=?utf-8?b?dsOkbMO8ZQ==?=')
-        self.assertEqual(actual, expected)
-
-    def test_plain_email_addresses_are_accepted(self):
-        address = 'user@example.com'
-        actual = utils.encode_header('from', address)
-        expected = email.header.Header(address)
-        self.assertEqual(actual, expected)
-
-    def test_email_addresses_with_realnames_are_accepted(self):
-        address = 'someone <user@example.com>'
-        actual = utils.encode_header('from', address)
-        expected = email.header.Header(address)
-        self.assertEqual(actual, expected)
-
-    def test_email_addresses_with_empty_realnames_are_treated_like_plain(self):
-        address = 'user@example.com'
-        empty_realname = '<'+address+'>'
-        actual = utils.encode_header('from', empty_realname)
-        expected = email.header.Header(address)
-        self.assertEqual(str(actual), str(expected))
-
-    def test_space_around_email_address_is_striped(self):
-        address = '  someone <user@example.com>  '
-        actual = utils.encode_header('from', address)
-        expected = email.header.Header(address.strip())
-        self.assertEqual(actual, expected)
-
-    def test_spaces_in_user_names_are_accepted(self):
-        address = 'some one <user@example.com>'
-        actual = utils.encode_header('from', address)
-        expected = email.header.Header(address)
-        self.assertEqual(actual, expected)
-
-    def test_multible_addresses_can_be_given(self):
-        addresses = 'one <guy@example.com>, other <guy@example.com>, ' \
-            'last <guy@example.com>'
-        actual = utils.encode_header('from', addresses)
-        expected = email.header.Header(addresses)
-        self.assertEqual(actual, expected)
-
-    def test_comma_in_names_are_allowed(self):
-        addresses = '"last, first" <guy@example.com>, ' \
-            '"name, other" <guy@example.com>'
-        actual = utils.encode_header('from', addresses)
-        expected = email.header.Header(addresses)
-        self.assertEqual(str(actual), str(expected))
-
-    def test_utf_8_chars_in_realnames_are_accepted(self):
-        address = u'Ümlaut <uemlaut@example.com>'
-        actual = utils.encode_header('from', address)
-        expected = email.header.Header(
-            '=?utf-8?q?=C3=9Cmlaut?= <uemlaut@example.com>')
-        self.assertEqual(actual, expected)
-
-
 class TestDecodeHeader(unittest.TestCase):
 
     @staticmethod
@@ -243,10 +177,10 @@ class TestDecodeHeader(unittest.TestCase):
         :rtype: str
         """
         string = unicode_string.encode(encoding)
-        output = '=?' + encoding + '?Q?'
+        output = b'=?' + encoding.encode('ascii') + b'?Q?'
         for byte in string:
-            output += '=' + byte.encode('hex').upper()
-        return output + '?='
+            output += b'=' + codecs.encode(bytes([byte]), 'hex').upper()
+        return (output + b'?=').decode('ascii')
 
     @staticmethod
     def _base64(unicode_string, encoding):
@@ -260,9 +194,10 @@ class TestDecodeHeader(unittest.TestCase):
         :rtype: str
         """
         string = unicode_string.encode(encoding)
-        b64 = base64.encodestring(string).strip()
-        return '=?' + encoding + '?B?' + b64 + '?='
-
+        b64 = base64.encodebytes(string).strip()
+        result_bytes = b'=?' + encoding.encode('utf-8') + b'?B?' + b64 + b'?='
+        result = result_bytes.decode('ascii')
+        return result
 
     def _test(self, teststring, expected):
         actual = utils.decode_header(teststring)
@@ -315,7 +250,11 @@ class TestDecodeHeader(unittest.TestCase):
             ' again: ' + self._quote(part, 'utf-8') + \
             ' latin1: ' + self._base64(part, 'iso-8859-1') + \
             ' and ' + self._quote(part, 'iso-8859-1')
-        expected = u'utf-8: ÄÖÜäöü again: ÄÖÜäöü latin1: ÄÖÜäöü and ÄÖÜäöü'
+        expected = (
+            u'utf-8: ÄÖÜäöü '
+            u'again: ÄÖÜäöü '
+            u'latin1: ÄÖÜäöü and ÄÖÜäöü'
+        )
         self._test(text, expected)
 
     def test_tabs_are_expanded_to_align_with_eigth_spaces(self):
@@ -377,18 +316,19 @@ class TestAddSignatureHeaders(unittest.TestCase):
 
         self.assertIn((utils.X_SIGNATURE_VALID_HEADER, u'True'), mail.headers)
         self.assertIn(
-            (utils.X_SIGNATURE_MESSAGE_HEADER, u'Untrusted: mocked'), mail.headers)
+            (utils.X_SIGNATURE_MESSAGE_HEADER, u'Untrusted: mocked'),
+            mail.headers)
 
     def test_unicode_as_bytes(self):
         mail = self.FakeMail()
         key = make_key()
-        key.uids = [make_uid('andreá@example.com',
-                             uid=u'Andreá'.encode('utf-8'))]
+        key.uids = [make_uid('andreá@example.com', uid=u'Andreá')]
         mail = self.check(key, True)
 
         self.assertIn((utils.X_SIGNATURE_VALID_HEADER, u'True'), mail.headers)
         self.assertIn(
-            (utils.X_SIGNATURE_MESSAGE_HEADER, u'Valid: Andreá'), mail.headers)
+            (utils.X_SIGNATURE_MESSAGE_HEADER, u'Valid: Andreá'),
+            mail.headers)
 
     def test_error_message_unicode(self):
         mail = self.check(mock.Mock(), mock.Mock(), u'error message')
@@ -397,13 +337,6 @@ class TestAddSignatureHeaders(unittest.TestCase):
             (utils.X_SIGNATURE_MESSAGE_HEADER, u'Invalid: error message'),
             mail.headers)
 
-    def test_error_message_bytes(self):
-        mail = self.check(mock.Mock(), mock.Mock(), b'error message')
-        self.assertIn((utils.X_SIGNATURE_VALID_HEADER, u'False'), mail.headers)
-        self.assertIn(
-            (utils.X_SIGNATURE_MESSAGE_HEADER, u'Invalid: error message'),
-            mail.headers)
-
     def test_get_key_fails(self):
         mail = self.FakeMail()
         with mock.patch('alot.db.utils.crypto.get_key',
@@ -433,7 +366,8 @@ class TestMessageFromFile(TestCaseClassCleanup):
                     with open(os.path.join(search_dir, each)) as f:
                         ctx.op_import(f)
 
-            cls.keys = [ctx.get_key("DD19862809A7573A74058FF255937AFBB156245D")]
+            cls.keys = [
+                ctx.get_key("DD19862809A7573A74058FF255937AFBB156245D")]
 
     def test_erase_alot_header_signature_valid(self):
         """Alot uses special headers for passing certain kinds of information,
@@ -442,13 +376,13 @@ class TestMessageFromFile(TestCaseClassCleanup):
         """
         m = email.message.Message()
         m.add_header(utils.X_SIGNATURE_VALID_HEADER, 'Bad')
-        message = utils.message_from_file(io.BytesIO(m.as_string()))
+        message = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIs(message.get(utils.X_SIGNATURE_VALID_HEADER), None)
 
     def test_erase_alot_header_message(self):
         m = email.message.Message()
         m.add_header(utils.X_SIGNATURE_MESSAGE_HEADER, 'Bad')
-        message = utils.message_from_file(io.BytesIO(m.as_string()))
+        message = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIs(message.get(utils.X_SIGNATURE_MESSAGE_HEADER), None)
 
     def test_plain_mail(self):
@@ -456,15 +390,15 @@ class TestMessageFromFile(TestCaseClassCleanup):
         m['Subject'] = 'test'
         m['From'] = 'me'
         m['To'] = 'Nobody'
-        message = utils.message_from_file(io.BytesIO(m.as_string()))
+        message = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertEqual(message.get_payload(), 'This is some text')
 
     def _make_signed(self):
         """Create a signed message that is multipart/signed."""
-        text = 'This is some text'
+        text = b'This is some text'
         t = email.mime.text.MIMEText(text, 'plain', 'utf-8')
         _, sig = crypto.detached_signature_for(
-            helper.email_as_string(t), self.keys)
+            t.as_bytes(policy=email.policy.SMTP), self.keys)
         s = email.mime.application.MIMEApplication(
             sig, 'pgp-signature', email.encoders.encode_7or8bit)
         m = email.mime.multipart.MIMEMultipart('signed', None, [t, s])
@@ -475,34 +409,35 @@ class TestMessageFromFile(TestCaseClassCleanup):
     def test_signed_headers_included(self):
         """Headers are added to the message."""
         m = self._make_signed()
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn(utils.X_SIGNATURE_VALID_HEADER, m)
         self.assertIn(utils.X_SIGNATURE_MESSAGE_HEADER, m)
 
     def test_signed_valid(self):
         """Test that the signature is valid."""
         m = self._make_signed()
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertEqual(m[utils.X_SIGNATURE_VALID_HEADER], 'True')
 
     def test_signed_correct_from(self):
         """Test that the signature is valid."""
         m = self._make_signed()
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         # Don't test for valid/invalid since that might change
-        self.assertIn('ambig <ambig@example.com>', m[utils.X_SIGNATURE_MESSAGE_HEADER])
+        self.assertIn(
+            'ambig <ambig@example.com>', m[utils.X_SIGNATURE_MESSAGE_HEADER])
 
     def test_signed_wrong_mimetype_second_payload(self):
         m = self._make_signed()
         m.get_payload(1).set_type('text/plain')
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('expected Content-Type: ',
                       m[utils.X_SIGNATURE_MESSAGE_HEADER])
 
     def test_signed_wrong_micalg(self):
         m = self._make_signed()
         m.set_param('micalg', 'foo')
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('expected micalg=pgp-...',
                       m[utils.X_SIGNATURE_MESSAGE_HEADER])
 
@@ -524,7 +459,7 @@ class TestMessageFromFile(TestCaseClassCleanup):
         """
         m = self._make_signed()
         m.set_param('micalg', 'PGP-SHA1')
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('expected micalg=pgp-',
                       m[utils.X_SIGNATURE_MESSAGE_HEADER])
 
@@ -539,7 +474,7 @@ class TestMessageFromFile(TestCaseClassCleanup):
         """
         m = self._make_signed()
         m.attach(email.mime.text.MIMEText('foo'))
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('expected exactly two messages, got 3',
                       m[utils.X_SIGNATURE_MESSAGE_HEADER])
 
@@ -551,9 +486,9 @@ class TestMessageFromFile(TestCaseClassCleanup):
         if signed:
             t = self._make_signed()
         else:
-            text = 'This is some text'
+            text = b'This is some text'
             t = email.mime.text.MIMEText(text, 'plain', 'utf-8')
-        enc = crypto.encrypt(t.as_string(), self.keys)
+        enc = crypto.encrypt(t.as_bytes(policy=email.policy.SMTP), self.keys)
         e = email.mime.application.MIMEApplication(
             enc, 'octet-stream', email.encoders.encode_7or8bit)
 
@@ -570,12 +505,12 @@ class TestMessageFromFile(TestCaseClassCleanup):
         # of the mail, rather than replacing the whole encrypted payload with
         # it's unencrypted equivalent
         m = self._make_encrypted()
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertEqual(len(m.get_payload()), 3)
 
     def test_encrypted_unsigned_is_decrypted(self):
         m = self._make_encrypted()
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         # Check using m.walk, since we're not checking for ordering, just
         # existence.
         self.assertIn('This is some text', [n.get_payload() for n in m.walk()])
@@ -585,13 +520,13 @@ class TestMessageFromFile(TestCaseClassCleanup):
         that there is a signature.
         """
         m = self._make_encrypted()
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertNotIn(utils.X_SIGNATURE_VALID_HEADER, m)
         self.assertNotIn(utils.X_SIGNATURE_MESSAGE_HEADER, m)
 
     def test_encrypted_signed_is_decrypted(self):
         m = self._make_encrypted(True)
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('This is some text', [n.get_payload() for n in m.walk()])
 
     def test_encrypted_signed_headers(self):
@@ -599,23 +534,24 @@ class TestMessageFromFile(TestCaseClassCleanup):
         there is a signature.
         """
         m = self._make_encrypted(True)
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn(utils.X_SIGNATURE_MESSAGE_HEADER, m)
-        self.assertIn('ambig <ambig@example.com>', m[utils.X_SIGNATURE_MESSAGE_HEADER])
+        self.assertIn(
+            'ambig <ambig@example.com>', m[utils.X_SIGNATURE_MESSAGE_HEADER])
 
     # TODO: tests for the RFC 2440 style combined signed/encrypted blob
 
     def test_encrypted_wrong_mimetype_first_payload(self):
         m = self._make_encrypted()
         m.get_payload(0).set_type('text/plain')
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('Malformed OpenPGP message:',
                       m.get_payload(2).get_payload())
 
     def test_encrypted_wrong_mimetype_second_payload(self):
         m = self._make_encrypted()
         m.get_payload(1).set_type('text/plain')
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('Malformed OpenPGP message:',
                       m.get_payload(2).get_payload())
 
@@ -625,7 +561,7 @@ class TestMessageFromFile(TestCaseClassCleanup):
         """
         s = self._make_signed()
         m = email.mime.multipart.MIMEMultipart('mixed', None, [s])
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn(utils.X_SIGNATURE_VALID_HEADER, m)
         self.assertIn(utils.X_SIGNATURE_MESSAGE_HEADER, m)
 
@@ -635,7 +571,7 @@ class TestMessageFromFile(TestCaseClassCleanup):
         """
         s = self._make_encrypted()
         m = email.mime.multipart.MIMEMultipart('mixed', None, [s])
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('This is some text', [n.get_payload() for n in m.walk()])
         self.assertNotIn(utils.X_SIGNATURE_VALID_HEADER, m)
         self.assertNotIn(utils.X_SIGNATURE_MESSAGE_HEADER, m)
@@ -647,7 +583,7 @@ class TestMessageFromFile(TestCaseClassCleanup):
         """
         s = self._make_encrypted(True)
         m = email.mime.multipart.MIMEMultipart('mixed', None, [s])
-        m = utils.message_from_file(io.BytesIO(m.as_string()))
+        m = utils.decrypted_message_from_file(io.StringIO(m.as_string()))
         self.assertIn('This is some text', [n.get_payload() for n in m.walk()])
         self.assertIn(utils.X_SIGNATURE_VALID_HEADER, m)
         self.assertIn(utils.X_SIGNATURE_MESSAGE_HEADER, m)
@@ -728,7 +664,8 @@ class TestExtractBody(unittest.TestCase):
     @mock.patch('alot.db.utils.settings.mailcap_find_match',
                 mock.Mock(return_value=(None, {'view': 'cat'})))
     def test_prefer_html(self):
-        expected = '<!DOCTYPE html><html><body>This is an html email</body></html>'
+        expected = (
+            '<!DOCTYPE html><html><body>This is an html email</body></html>')
         mail = self._make_mixed_plain_html()
         actual = utils.extract_body(mail)
 
@@ -755,7 +692,8 @@ class TestExtractBody(unittest.TestCase):
             '<!DOCTYPE html><html><body>This is an html email</body></html>',
             'html'))
         actual = utils.extract_body(mail)
-        expected = '<!DOCTYPE html><html><body>This is an html email</body></html>'
+        expected = (
+            '<!DOCTYPE html><html><body>This is an html email</body></html>')
 
         self.assertEqual(actual, expected)
 
@@ -768,18 +706,19 @@ class TestExtractBody(unittest.TestCase):
             '<!DOCTYPE html><html><body>This is an html email</body></html>',
             'html'))
         actual = utils.extract_body(mail)
-        expected = '<!DOCTYPE html><html><body>This is an html email</body></html>'
+        expected = (
+            '<!DOCTYPE html><html><body>This is an html email</body></html>')
 
         self.assertEqual(actual, expected)
 
 
 class TestMessageFromString(unittest.TestCase):
 
-    """Tests for message_from_string.
+    """Tests for decrypted_message_from_string.
 
     Because the implementation is that this is a wrapper around
-    message_from_file, it's not important to have a large swath of tests, just
-    enough to show that things are being passed correctly.
+    decrypted_message_from_file, it's not important to have a large swath of
+    tests, just enough to show that things are being passed correctly.
     """
 
     def test(self):
@@ -787,5 +726,42 @@ class TestMessageFromString(unittest.TestCase):
         m['Subject'] = 'test'
         m['From'] = 'me'
         m['To'] = 'Nobody'
-        message = utils.message_from_string(m.as_string())
+        message = utils.decrypted_message_from_string(m.as_string())
         self.assertEqual(message.get_payload(), 'This is some text')
+
+
+class TestRemoveCte(unittest.TestCase):
+
+    def test_char_vs_cte_mismatch(self):  # #1291
+        with open('tests/static/mail/broken-utf8.eml') as fp:
+            mail = email.message_from_file(fp)
+        # This should not raise an UnicodeDecodeError.
+        with self.assertLogs(level='DEBUG') as cm:  # keep logs
+            utils.remove_cte(mail, as_string=True)
+        # We expect no Exceptions but a complaint in the log
+        logmsg = 'DEBUG:root:Decoding failure: \'utf-8\' codec can\'t decode '\
+            'byte 0xa1 in position 14: invalid start byte'
+        self.assertIn(logmsg, cm.output)
+
+    def test_malformed_cte_value(self):
+        with open('tests/static/mail/malformed-header-CTE.eml') as fp:
+            mail = email.message_from_file(fp)
+
+        with self.assertLogs(level='INFO') as cm:  # keep logs
+            utils.remove_cte(mail, as_string=True)
+
+        # We expect no Exceptions but a complaint in the log
+        logmsg = 'INFO:root:Unknown Content-Transfer-Encoding: "7bit;"'
+        self.assertEqual(cm.output, [logmsg])
+
+    def test_unknown_cte_value(self):
+        with open('tests/static/mail/malformed-header-CTE-2.eml') as fp:
+            mail = email.message_from_file(fp)
+
+        with self.assertLogs(level='DEBUG') as cm:  # keep logs
+            utils.remove_cte(mail, as_string=True)
+
+        # We expect no Exceptions but a complaint in the log
+        logmsg = 'DEBUG:root:failed to interpret Content-Transfer-Encoding: '\
+                 '"normal"'
+        self.assertIn(logmsg, cm.output)
